-- scope stuff
qualityoflife = qualityoflife or {};
local q = qualityoflife;

q.modules = q.modules or {};
q.modules["QoL_ClassicHeals"] = q.modules["QoL_ClassicHeals"] or {};
local c = q.modules["QoL_ClassicHeals"];

function c:Println(str)
	if str then
		q:Println(str, "ClassicHeals");
	end
end

function c:GetAffectedPartyMemberId(memberName)
	for playerId, playerName in ipairs(q:GetPartyMembers()) do
		if playerName == memberName then
			return playerId;
		end
	end
end

function c:GetAffectedPartyUnitId(target)
	target = q:RemoveRealmFromName(target);
	for partyId = 1, 5 do
		local unitId = "party" .. partyId;
		if q:RemoveRealmFromName(GetUnitName(unitId)) == target then
			return unitId;
		end

		local unitPetId = "partypet" .. partyId;
		if q:RemoveRealmFromName(GetUnitName(unitPetId)) == target then
			return unitPetId;
		end
	end
end

function c:GetAffectedRaidMemberId(memberName)
	local rm = q:GetRaidMembers();
	local raidId;

	for playerId, playerName in ipairs(rm[0]) do
		if playerName == memberName then
			raidId = playerId;
			break;
		end
	end

	for subGroupId, subGroup in pairs(rm) do
		if subGroupId > 0 then
			for playerInSubGroupId, playerName in ipairs(rm[subGroupId]) do
				if playerName == memberName then
					return raidId, subGroupId, playerInSubGroupId;
				end;
			end;
		end
	end

	if memberName == GetUnitName("player") then
		return 1, 1, 1; -- for grid, vuhdo etc.
	end
end

function c:GetAffectedRaidUnitId(target)
	target = q:RemoveRealmFromName(target);
	for raidId = 1, 40 do
		local unitId = "raid" .. raidId;
		if q:RemoveRealmFromName(GetUnitName(unitId)) == target then
			return unitId;
		end

		local unitPetId = "raidpet" .. raidId;
		if q:RemoveRealmFromName(GetUnitName(unitPetId)) == target then
			return unitPetId;
		end
	end
end

-- the bruteforceish approaches seem to work best
local function GetCompactPartyFrame(target)
	for n = 1, 10 do -- pets increase the number of frames (not confirmed)
		local compactPartyFrameName = "CompactPartyFrameMember" .. n;
		local compactPartyFrame = _G[compactPartyFrameName];
		if compactPartyFrame and compactPartyFrame:IsVisible() and _G[compactPartyFrameName .. "Name"] and _G[compactPartyFrameName .. "Name"]:GetText() == target then
			return compactPartyFrame;
		end
	end
end

local function GetCompactRaidFrame(target)
	for n = 1, 80 do -- pets increase the number of frames
		local compactRaidFrameName = "CompactRaidFrame" .. n;
		local compactRaidFrame = _G[compactRaidFrameName];
		if compactRaidFrame and compactRaidFrame:IsVisible() and _G[compactRaidFrameName .. "Name"] and _G[compactRaidFrameName .. "Name"]:GetText() == target then
			return compactRaidFrame;
		end
	end
end

local function GetCompactRaidGroupFrame(target)
	for groupId = 1, 8 do
		for subId = 1, 5 do
			local compactRaidGroupFrameName = "CompactRaidGroup" .. groupId .. "Member" .. subId;
			local compactRaidGroupFrame = _G[compactRaidGroupFrameName];
			if compactRaidGroupFrame and compactRaidGroupFrame:IsVisible() and _G[compactRaidGroupFrameName .. "Name"] and _G[compactRaidGroupFrameName .. "Name"]:GetText() == target then
				return compactRaidGroupFrame;
			end
		end
	end
end

local function GetRaidPulloutFrame(target)
	for groupId = 1, 8 do
		for subId = 1, 5 do
			local raidPulloutName = "RaidPullout" .. groupId .. "Button" .. subId;
			if _G[raidPulloutName] and _G[raidPulloutName .. "Name"] and _G[raidPulloutName .. "Name"]:GetText() == target then
				return _G[raidPulloutName];
			end
		end
	end
end

local function GetVuhdoFrameNames(target)
	local result = {};
	if VUHDO_PANEL_UNITS then
		-- kind of a brute force attempt. but the vuhdo code is way too obfuscated
		for playerId = 1, 80 do -- seems like pets increase the number of frames...
			local frameName = "Vd2H" .. playerId;
			if _G[frameName] and _G[frameName].raidid and _G[frameName].raidid == target then
				tinsert(result, frameName);
			end
		end
	end
	return result;
end

local function GetVuhdoRaidFrameNames(target, result)
	if not result then
		result = {};
	end

	if VUHDO_PANEL_UNITS then
		-- kind of a brute force attempt. but the vuhdo code is way too obfuscated
		for playerId = 1, 80 do -- seems like pets increase the number of frames...
			local frameName = "Vd1H" .. playerId;
			if _G[frameName] and _G[frameName].raidid and _G[frameName].raidid == target then
				tinsert(result, frameName);
			end
		end
	end
	return result;
end

local function GetGrid2FrameNames(target)
	local result = {};

	-- just grid2 stuff
	target = q:RemoveRealmFromName(target);
	if strlen(target) > 6 then
		target = strsub (target, 1, 6);
	end

	if _G["Grid2LayoutFrame"] then
		for subGroupId = 1, 8 do
			for playerId = 1, 5 do
				local frameName = "Grid2LayoutHeader" .. subGroupId .. "UnitButton" .. playerId;
				-- some kind of reverse engineering. seems to be working
				if _G[frameName] and _G[frameName]["text-down"] and _G[frameName]["text-down"].Text and _G[frameName]["text-down"].Text.GetText and _G[frameName]["text-down"].Text:GetText() == target then
					tinsert(result, frameName);
				end
			end
		end
	end
	return result;
end

function c:GetTargetFrames(target)
	local result, partyMemberId, raidId, subGroupId, playerInSubGroupId = {}, c:GetAffectedPartyMemberId(target), c:GetAffectedRaidMemberId(target);

	----- default wow / elvUI -----
	if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.OPT_UIATTR_ENABLED] then
		result[c.PROFILE_DEFAULT] = {};

		-- player frames
		if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PLAYER][c.OPT_UIATTR_ENABLED] and target == GetUnitName("player") then
			result[c.PROFILE_DEFAULT][c.PROFILE_PLAYER] = {};

			-- default player frame
			if _G["PlayerFrame"] and _G["PlayerFrame"]:IsVisible() then
				table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_PLAYER], _G["PlayerFrame"]);
			end

			-- elvui player frame
			if _G["ElvUF_Player"] and _G["ElvUF_Player"]:IsVisible() then
				table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_PLAYER], _G["ElvUF_Player"]);
			end
		end

		-- target frames
		if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGET][c.OPT_UIATTR_ENABLED] and target == GetUnitName("target") then
			result[c.PROFILE_DEFAULT][c.PROFILE_TARGET] = {};

			-- default target frame
			if _G["TargetFrame"] and _G["TargetFrame"]:IsVisible() then
				table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_TARGET], _G["TargetFrame"]);
			end

			-- elvui target frame
			if _G["ElvUF_Target"] and _G["ElvUF_Target"]:IsVisible() then
				table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_TARGET], _G["ElvUF_Target"]);
			end
		end

		-- party frames
		if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PARTY][c.OPT_UIATTR_ENABLED] and partyMemberId then
			result[c.PROFILE_DEFAULT][c.PROFILE_PARTY] = {};

			-- default party frame
			local defaultPartyFrameName = "PartyMemberFrame" .. partyMemberId;
			if _G[defaultPartyFrameName] and _G[defaultPartyFrameName]:IsVisible() then
				table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_PARTY], _G[defaultPartyFrameName]);
			end

			-- elvUI party frame
			local elvUiPartyFrameName = "ElvUF_PartyGroup1UnitButton" .. partyMemberId + 1;
			if _G[elvUiPartyFrameName] and _G[elvUiPartyFrameName]:IsVisible() then
				table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_PARTY], _G[elvUiPartyFrameName]);
			end
		end
	end

	-- raid frames
	if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] or QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT][c.OPT_UIATTR_ENABLED] then
		if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] then
			result[c.PROFILE_DEFAULT][c.PROFILE_RAID_COMPACT] = {};

			-- default compactraidgroup / compactraidframe
			if _G["CompactRaidFrameContainer"]:IsVisible() then
				local compactRaidFrame, compactRaidGroupFrame = GetCompactRaidFrame(target), GetCompactRaidGroupFrame(target);
				if compactRaidFrame then
					table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_RAID_COMPACT], compactRaidFrame);
				end
				if compactRaidGroupFrame then
					table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_RAID_COMPACT], compactRaidGroupFrame);
				end

				-- compact party frame (compact raid frame style)
				local compactPartyFrame = GetCompactPartyFrame(target);
				if compactPartyFrame then
					table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_RAID_COMPACT], compactPartyFrame);
				end
			end
		end

		if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT][c.OPT_UIATTR_ENABLED] and subGroupId and playerInSubGroupId then
			result[c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT] = {};

			-- default raid pull out group
			local pulloutFrame = GetRaidPulloutFrame(target);
			if pulloutFrame then
				table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT], pulloutFrame);
			end
		end
	end

	----- addons -----

	-- luna
	if c:GetProfile(c.PROFILE_ADDON_LUNA) and QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.OPT_UIATTR_ENABLED] then
		result[c.PROFILE_ADDON_LUNA] = {};

		-- player frame
		if QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.PROFILE_PLAYER][c.OPT_UIATTR_ENABLED] and target == GetUnitName("player") then
			result[c.PROFILE_ADDON_LUNA][c.PROFILE_PLAYER] = {};

			if _G["LUFUnitplayer"] and _G["LUFUnitplayer"]:IsVisible() then
				table.insert(result[c.PROFILE_ADDON_LUNA][c.PROFILE_PLAYER], _G["LUFUnitplayer"].highFrame);
			end
		end

		-- target frame
		if QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.PROFILE_TARGET][c.OPT_UIATTR_ENABLED] and target == GetUnitName("target") then
			result[c.PROFILE_ADDON_LUNA][c.PROFILE_TARGET] = {};

			if _G["LUFUnittarget"] and _G["LUFUnittarget"]:IsVisible() then
				table.insert(result[c.PROFILE_ADDON_LUNA][c.PROFILE_TARGET], _G["LUFUnittarget"].highFrame);
			end
		end

		-- party frame
		if partyMemberId and QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.PROFILE_PARTY][c.OPT_UIATTR_ENABLED] then
			result[c.PROFILE_ADDON_LUNA][c.PROFILE_PARTY] = {};

			local lunaFrameName = "LUFHeaderpartyUnitButton" .. partyMemberId;
			if _G[lunaFrameName] and _G[lunaFrameName]:IsVisible() then
				table.insert(result[c.PROFILE_ADDON_LUNA][c.PROFILE_PARTY], _G[lunaFrameName]);
			end
		end

		-- raid
		if subGroupId and playerInSubGroupId and QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] then
			result[c.PROFILE_ADDON_LUNA][c.PROFILE_RAID_COMPACT] = {};

			local lunaFrameName = "LUFHeaderraid" .. subGroupId .. "UnitButton" .. playerInSubGroupId;
			if _G[lunaFrameName] and _G[lunaFrameName]:IsVisible() then
				table.insert(result[c.PROFILE_ADDON_LUNA][c.PROFILE_RAID_COMPACT], _G[lunaFrameName]);
			end
		end
	end

	-- grid2
	if c:GetProfile(c.PROFILE_ADDON_GRID2) and subGroupId and playerInSubGroupId and QOL_OPTIONS[c.name][c.PROFILE_ADDON_GRID2][c.OPT_UIATTR_ENABLED] and QOL_OPTIONS[c.name][c.PROFILE_ADDON_GRID2][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] then
		result[c.PROFILE_ADDON_GRID2] = {};
		result[c.PROFILE_ADDON_GRID2][c.PROFILE_RAID_COMPACT] = {};

		local gridFrameNames = GetGrid2FrameNames(target);
		for _, frameName in ipairs(gridFrameNames) do
			if _G[frameName] and _G[frameName]:IsVisible() then
				table.insert(result[c.PROFILE_ADDON_GRID2][c.PROFILE_RAID_COMPACT], _G[frameName]);
			end
		end
	end

	-- vuhdo
	if c:GetProfile(c.PROFILE_ADDON_VUHDO) and QOL_OPTIONS[c.name][c.PROFILE_ADDON_VUHDO][c.OPT_UIATTR_ENABLED] then
		result[c.PROFILE_ADDON_VUHDO] = {};

		-- target
		if QOL_OPTIONS[c.name][c.PROFILE_ADDON_VUHDO][c.PROFILE_TARGET][c.OPT_UIATTR_ENABLED] and UnitName("target") then
			result[c.PROFILE_ADDON_VUHDO][c.PROFILE_TARGET] = {};

			local vuhdoTargetFrameNames = GetVuhdoFrameNames("target");
			for _, frameName in ipairs(vuhdoTargetFrameNames) do
				if frameName and _G[frameName .. "BgBarIcBarHlBarIc"] and _G[frameName .. "BgBarIcBarHlBarIc"]:IsVisible() then
					table.insert(result[c.PROFILE_ADDON_VUHDO][c.PROFILE_TARGET], _G[frameName .. "BgBarIcBarHlBarIc"]);
				end
			end
		end

		if (target == GetUnitName("player") or target == "player") and QOL_OPTIONS[c.name][c.PROFILE_ADDON_VUHDO][c.PROFILE_PLAYER][c.OPT_UIATTR_ENABLED] then
			result[c.PROFILE_ADDON_VUHDO][c.PROFILE_PLAYER] = {};
			-- player frame
			local vuhdoRaidFrameNames = GetVuhdoFrameNames("player");
			for _, frameName in ipairs(vuhdoRaidFrameNames) do
				if frameName and _G[frameName .. "BgBarIcBarHlBarIc"] and _G[frameName .. "BgBarIcBarHlBarIc"]:IsVisible() then
					table.insert(result[c.PROFILE_ADDON_VUHDO][c.PROFILE_PLAYER], _G[frameName .. "BgBarIcBarHlBarIc"]);
				end
			end
		end

		if QOL_OPTIONS[c.name][c.PROFILE_ADDON_VUHDO][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] then
			result[c.PROFILE_ADDON_VUHDO][c.PROFILE_RAID_COMPACT] = {};
			-- raid frame
			local vuhdoRaidFrameNames = GetVuhdoRaidFrameNames(c:GetAffectedRaidUnitId(target));
			vuhdoRaidFrameNames = GetVuhdoRaidFrameNames(c:GetAffectedPartyUnitId(target), vuhdoRaidFrameNames);
			if (target == GetUnitName("player") or target == "player") then
				vuhdoRaidFrameNames = GetVuhdoRaidFrameNames("player", vuhdoRaidFrameNames);
			end

			for _, frameName in ipairs(vuhdoRaidFrameNames) do
				if frameName and _G[frameName .. "BgBarIcBarHlBarIc"] and _G[frameName .. "BgBarIcBarHlBarIc"]:IsVisible() then
					table.insert(result[c.PROFILE_ADDON_VUHDO][c.PROFILE_RAID_COMPACT], _G[frameName .. "BgBarIcBarHlBarIc"]);
				end
			end
		end
	end

	return result;
end

function c:ShowResetDialog()
	local dialogName = c.name .. "_RESETDIALOG";
	StaticPopupDialogs[dialogName] = StaticPopupDialogs[dialogName] or {
		text = q:GetText("This will reset all ClassicHeals settings to defaults.\n\nAre you sure?"),
		button1 = q:GetText("Reset"),
		button2 = q:GetText("Cancel"),
		OnEscapePressed = function ()
			self:Hide();
		end,
		OnAccept = function()
			QOL_OPTIONS[c.name] = {};
			c:InitProfiles(true);
			c:InitOptions();
			c:SaveSettingsChanges();
			c:SetProfileValues();
		end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		preferredIndex = 3
	};
	StaticPopup_Show(dialogName);
end